//-------------------------------------------------------------------------//
// Copyright (c) Raymarine UK Limited 2019
//
// Reproduction or transmission in whole or in part (whether by photocopying or
// storing in any medium by electronic means or otherwise) without the written
// permission of Raymarine UK Limited is prohibited.
//
// Confidential
//-------------------------------------------------------------------------//

#pragma once

//-------------------------------------------------------------------------//

#include "IProcessWatchdog.h"
#include "time.h"

//-------------------------------------------------------------------------//

class ProcessWatchdogMonitor : public IProcessWatchdog
{
public:
    /// Constructor
    ProcessWatchdogMonitor();
    virtual ~ProcessWatchdogMonitor();

protected:
    ///
    /// \brief ProcessWatchdogMonitor::_reset
    /// Resets the worst time, should be called when the watchdog timeout time
    /// is changed
    ///
    /// \param Timeout
    ///
    void _reset(int timeout);

    ///
    /// \brief ProcessWatchdogMonitor::_canToggle
    ///
    /// Determines if it is time to toggle the watchdog and updates the monitor times
    ///
    /// \param currentLateTime_ms   - set to the number of milliseconds that have passed since this toggle should have occurred
    /// \param worstLateTime_ms     - set the worst late toggle time
    ///
    /// \return true if it is time to toggle the watchdog
    ///
    bool _canToggle(long long &currentLateTime_ms, long long &worstLateTime_ms);

    ///
    /// \brief ProcessWatchdogMonitor::_updateToggleTime
    ///
    /// Sets the toggle time to the current time.
    ///
    void _updateToggleTime();

    virtual unsigned int getToggleRate() const = 0;

private:
    struct timespec m_lastToggleTime;               // Last time the watchdog was toggled

    unsigned long m_Timeout_ms;                     // Current watchdog timeout
    unsigned long m_worstWatchdogUpdateTime_ms;     // worst watchdog toggle time
};

