/*
Copyright (c) 2013, Raymarine UK Limited
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Raymarine UK Limited nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _RAYASSERT_H
#define _RAYASSERT_H

#include <features.h>

#if defined __cplusplus // && __GNUC_PREREQ (2,95)
    # define __ASSERT_VOID_CAST static_cast<void>
#else
    # define __ASSERT_VOID_CAST (void)
#endif

#ifdef NDEBUG
    #define RAYASSERT(Cond) do { __ASSERT_VOID_CAST (Cond); } while (false)
#else
    #ifdef TARGET_ANDROID
        #include <assert.h>
        #define RAYASSERT assert // NOLINT
    #elif TARGET_DESKTOP
        #ifdef APOLLO_UNIT_TEST
            #ifdef __cplusplus
                extern "C" void test_assert(const char *cond, const char* file, int line);
            #else
                extern void test_assert(const char *cond, const char* file, int line);
            #endif
            #define RAYASSERT(Cond) do { if (!(Cond)) test_assert(#Cond, __FILE__, __LINE__); } while (false)
        #else
            #include <assert.h>
            #define RAYASSERT assert // NOLINT
        #endif
    #elif RAY_UNIT_TEST
        #define RAYASSERT(Cond) do { __ASSERT_VOID_CAST (Cond); } while (false)

    #else // !RAY_UNIT_TEST
        #ifdef _NO_DIAGNOSTICS
            #include <assert.h>
            #define RAYASSERT assert
        #else
            #ifdef __cplusplus
                extern "C" void eCrash_assert(const char *cond, const char* file, int line) __THROW __attribute__ ((__noreturn__));
            #else
                extern void eCrash_assert(const char *cond, const char* file, int line) __THROW __attribute__ ((__noreturn__));
            #endif
            #define RAYASSERT(Cond) do { if (!(Cond)) eCrash_assert(#Cond, __FILE__, __LINE__); } while (false)
        #endif
    #endif
#endif

#endif
