﻿/*
Copyright (c) 2016, Raymarine UK Limited
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Raymarine UK Limited nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef DISCOVERYSERVICE_H_INCLUDED
#define DISCOVERYSERVICE_H_INCLUDED

#include <QThread>
#include <set>
#include <string>

class CIpCameraStore;

// Discovery service for the daemon.  Does probes to find cameras on the network
// and emits the ProbeMatch signal when it finds them
class CIpInfo;
class CDiscoveryService : public QThread
{
    Q_OBJECT

public:
    CDiscoveryService();
    virtual ~CDiscoveryService();

    // Public slot called by the gSoap ProbeMatch event handler when it gets a match
    void GotProbeMatch(const std::string& OnvifNode);

    // Signal emitted for the IP Video Control when we get a probe match
    Q_SIGNAL void ProbeMatch(const std::string& OnvifNode);

    // The C functions also need to be able to get it
    static std::string m_LocalIpAddress;
    static std::string m_LocalIpAddressLegacy;
protected:
    void run();
    void _runImpl(bool newSubnet = true);

private:
    Q_SLOT void _DoProbe(bool newSubnet = true);

    void _Init(bool newSubnet = true);

    struct soap* m_pSoap;          // Class soap structure
    struct soap* m_pSoapLegacy;          // Class soap structure
    bool m_KeepRunning;            // Flag for QThread run function loop
    std::set<std::string> m_Nodes;

    CIpInfo* m_pIpInfo;
};

#endif // DISCOVERYSERVICE_H_INCLUDED
