//-------------------------------------------------------------------------//
// Copyright (c) Raymarine UK Limited 2016
//
// Reproduction or transmission in whole or in part (whether by photocopying or
// storing in any medium by electronic means or otherwise) without the written
// permission of Raymarine UK Limited is prohibited.
//
// Confidential
//-------------------------------------------------------------------------//

#ifndef NETWORKHELPERS_H
#define NETWORKHELPERS_H

#include <QLocalSocket>
#include <QDataStream>
#include <QVariant>

enum ServerResponsePacketHeader
{
    ServerResponse_CameraFound,
    ServerResponse_CameraLost,
    ServerResponse_ParameterChanged
};

enum ClientRequestPacketHeader
{
    ClientRequest_SendAllCameras,
    ClientRequest_SetContrast,
    ClientRequest_SetBrightness,
    ClientRequest_SetColour,
    ClientRequest_SetName,
    ClientRequest_EnableCameraAutoConfiguration
};

template <typename T>
void readPackedFromBuffer(QByteArray& buffer, T interpretPacketFunction)
{
    QDataStream serializer(buffer);

    // Check if we have enough to read size (quint32) of the packet and at least 1 byte of the data
    if (buffer.size() > 4)
    {
        qint32 packedSize;
        serializer >> packedSize;
        // Now we check if we have enough data to read whole packet
        if (buffer.size() >= packedSize)
        {
            qint8 header;
            serializer >> header;

            // Now that we have header and full data we can interpret it
            interpretPacketFunction(header, serializer);

            // After interpreting it we can remove data from our buffer
            buffer.remove(0, packedSize);
        }
    }
}

inline void serializeObjects(QDataStream&)
{
}

template<typename T, typename... ARGS>
void serializeObjects(QDataStream& dataStream, T& object, ARGS&... objects)
{
    dataStream << object;
    serializeObjects(dataStream, objects...);
}

template <typename T, typename... ARGS>
QByteArray createPacket(quint8 header, T& object, ARGS&... objects)
{
    QByteArray objectByteArray;
    QDataStream objectDataStream(&objectByteArray, QIODevice::WriteOnly);
    serializeObjects(objectDataStream, object, objects...);

    quint32 packetSize = sizeof(quint32) + sizeof(quint8) + static_cast<quint32>(objectByteArray.size());

    QByteArray packetByteArray;
    QDataStream packetDataStream(&packetByteArray, QIODevice::WriteOnly);

    packetDataStream << packetSize;
    packetDataStream << header;
    packetDataStream.writeRawData(objectByteArray.data(), objectByteArray.size());

    return packetByteArray;
}

inline QByteArray createPacket(quint8 header)
{
    quint32 packetSize = sizeof(quint32) + sizeof(quint8);

    QByteArray packetByteArray;
    QDataStream packetDataStream(&packetByteArray, QIODevice::WriteOnly);

    packetDataStream << packetSize;
    packetDataStream << header;

    return packetByteArray;
}

#endif // NETWORKHELPERS_H
