/*
Copyright (c) 2016, Raymarine UK Limited
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Raymarine UK Limited nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <cstdio>
#include <iomanip>
#include <signal.h>
#include <sys/prctl.h>
#include <unistd.h>
#include <sstream>


#include "Framework/PlatformAbstraction/RayService.h"
#include "IPVideoControl.h"

#include "FrameworkCommon/IProcessWatchdog.h"
#ifdef __ANDROID__
#include "FrameworkCommon/ProcessWatchdogAndroid.h"
#else
#include "FrameworkCommon/ProcessWatchdogFifoFile.h"
#endif
#ifdef TARGET_ATOM
#include "FrameworkCommon/CrashLogConfiguration.h"
#include "FrameworkCommon/OMAPCrash.h"
#endif

// Crash log configuration
static const char* c_BackTraceFolder    = "/raymarine/Data/Log";
static const int   c_MaximumLogFiles    = 50;
static const int   c_MaximumScreenDumps = 0;
static const char* c_LogFilenameBase    = "ONVIFDAEMON";
static const int   c_NormalWatchdogTimeout = 30;

int main(int argc, char** argv)
{
    // Ensure we get killed if our parent dies.
    prctl(PR_SET_PDEATHSIG, SIGKILL);

    // Create a QApplication object.
    RayService app(argc, argv);

#ifdef __ANDROID__
    CProcessWatchdogAndroid Watchdog;
#else
    CProcessWatchdogFifoFile Watchdog("/tmp/OnvifDaemon.Watchdog");
#endif
#ifdef TARGET_ATOM
    // Convert build ID macro to a string.
    std::stringstream SoftwareVersionString;
    SoftwareVersionString << "v" << _AUTO_BUILD_MAJOR_ << "." << _AUTO_BUILD_MINOR_ << "." << _AUTO_BUILD_ID_;

    // Configure the process for crash logging.
    CCrashLogConfiguration  CrashLogConfiguration;
    CrashLogConfiguration.Init(c_BackTraceFolder,
                               c_LogFilenameBase,
                               argv[0],
                               SoftwareVersionString.str().c_str(),
                               c_MaximumLogFiles,
                               c_MaximumScreenDumps);
#endif
    Watchdog.SetWatchdogTimeout(c_NormalWatchdogTimeout);

    // Create the top-level object.

    CIPVideoControl VideoControl(&Watchdog);

    // Start the Qt message loop.
    int Ret = app.exec();

#ifdef __ANDROID__
    // Restarting services doesn't work well under Android as static objects are not destroyed and recreated as the process doesn't get killed!!!
    kill(getpid(), SIGKILL);
#endif
    return Ret;
}
